@@ -13,7 +13,7 @@ from pandas.api.types import is_numeric_dtype
 from abc import ABC, abstractmethod
 import abc
 import loompy
-
+from pathlib import Path
 BBROWSER_VERSION = "2.7.38"
 DEFAULT_BARCODE_NAME = ["index", "_index", "CellID", "observation_id"]
 DEFAULT_FEATURE_NAME = ["index", "_index", "Gene", "accession_id"]
@@ -410,30 +410,16 @@ class DataObject(ABC):
         Returns:
             None
         """
-        print("Writing group \"bioturing\"")
-        bioturing_group = dest_hdf5.create_group("bioturing")
-        bioturing_group.create_dataset("barcodes",
-                                        data=encode_strings(barcodes))
-        bioturing_group.create_dataset("features",
-                                        data=encode_strings(features))
-        bioturing_group.create_dataset("data", data=raw_M.data)
-        bioturing_group.create_dataset("indices", data=raw_M.indices)
-        bioturing_group.create_dataset("indptr", data=raw_M.indptr)
-        bioturing_group.create_dataset("feature_type", data=["RNA".encode("utf8")] * len(features))
-        bioturing_group.create_dataset("shape", data=[len(features), len(barcodes)])
-
         if has_raw:
             print("Writing group \"countsT\"")
             raw_M_T = raw_M.tocsc()
             countsT_group = dest_hdf5.create_group("countsT")
-            countsT_group.create_dataset("barcodes",
-                                            data=encode_strings(features))
-            countsT_group.create_dataset("features",
-                                            data=encode_strings(barcodes))
             countsT_group.create_dataset("data", data=raw_M_T.data)
-            countsT_group.create_dataset("indices", data=raw_M_T.indices)
-            countsT_group.create_dataset("indptr", data=raw_M_T.indptr)
-            countsT_group.create_dataset("shape", data=[len(barcodes), len(features)])
+            if np.array_equal(raw_M_T.indptr, norm_M.indptr) == False \
+                or np.array_equal(raw_M_T.indices, norm_M.indices) == False:
+                print("Raw data and normalized data do not share the same indptr and indices")
+                countsT_group.create_dataset("indices", data=raw_M_T.indices);
+                countsT_group.create_dataset("indptr", data=raw_M_T.indptr);
         else:
             print("Raw data is not available, ignoring \"countsT\"")
 
@@ -457,12 +443,14 @@ class DataObject(ABC):
             sum_raw = np.array([0.0] * n_cells)
 
         for i in range(n_cells):
-            l, r = raw_M.indptr[i:i+2]
-            sum_lognorm[i] = np.sum(norm_M.data[l:r])
             if has_raw:
+                l, r = raw_M.indptr[i:i+2]
                 sum_raw[i] = np.sum(raw_M.data[l:r])
                 sum_log[i] = np.sum(np.log2(raw_M.data[l:r] + 1))
 
+            l, r = norm_M.indptr[i:i+2]
+            sum_lognorm[i] = np.sum(norm_M.data[l:r])
+
         colsum_group = dest_hdf5.create_group("colsum")
         colsum_group.create_dataset("lognorm", data=sum_lognorm)
         if has_raw:
@@ -574,6 +562,16 @@ class DataObject(ABC):
         with zobj.open(os.path.join(runinfo_path, "run_info.json"), "w") as z:
             z.write(json.dumps(run_info).encode("utf8"))
 
+    def write_bcs_info(self, zobj, bcs_info_path):
+        version = {}
+        with open(os.path.join(Path(__file__).parent, "version.py"), "r") as f:
+            exec(f.read(), version)
+        bcs_info = {
+                "version":version["__version__"]
+                }
+        with zobj.open(os.path.join(bcs_info_path, "bcs_info.json"), "w") as z:
+            z.write(json.dumps(bcs_info).encode("utf8"))
+
     def write_bcs_to_file(self, zobj, study_name, replace_missing):
         """Write data to a given zobj file as bcs format
 
@@ -594,6 +592,7 @@ class DataObject(ABC):
         self.write_runinfo(zobj, study_name=study_name,
                             runinfo_path=self.get_runinfo_path(study_name),
                             unit=unit)
+        self.write_bcs_info(zobj, bcs_info_path=study_name)
 
     def write_bcs(self, study_name, output_name, replace_missing="Unassigned"):
         """Writes data to bcs file
@@ -665,12 +664,12 @@ class ScanpyData(DataObject):
 
     def get_raw_matrix(self):
         try:
-            return self.object.raw.X[:][:].tocsr()
+            return scipy.sparse.csr_matrix(self.object.raw.X[:][:])
         except:
-            return self.object.layers[self.raw_key].tocsr()
+            return scipy.sparse.csr_matrix(self.object.layers[self.raw_key])
 
     def get_normalized_matrix(self):
-        return self.object.X[:][:].tocsc()
+        return scipy.sparse.csc_matrix(self.object.X[:][:])
 
     def get_metadata(self):
         return self.object.obs
